'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.plugin = plugin;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _path = require('path');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsUtils = require('./fs-utils');

var _generators = require('./generators');

var symb = Symbol('dts-meta');

function withMeta(visitFn) {
  return function visit(node) {
    var meta = this.state.metadata[symb]; // eslint-disable-line no-invalid-this
    return visitFn(node, meta);
  };
}

var skipper = {
  enter: withMeta(function (node, _ref) {
    var moduleExports = _ref.moduleExports;
    var skipStack = _ref.skipStack;

    if (skipStack.length === 0) {
      moduleExports.push(node);
    }

    skipStack.push(node);
  }),

  exit: withMeta(function (node, _ref2) {
    var skipStack = _ref2.skipStack;

    if (node !== skipStack.pop()) {
      throw new Error('skipStack unballance');
    }
  })
};

function plugin(_ref3) {
  var Plugin = _ref3.Plugin;
  var t = _ref3.types;
  // eslint-disable-line
  return new Plugin('aurelia-babel-plugin', {
    visitor: {
      ExportAllDeclaration: withMeta(function (node, _ref4) {
        var moduleExports = _ref4.moduleExports;

        moduleExports.push(node);
      }),

      ExportNamedDeclaration: skipper,

      ExportDefaultDeclaration: skipper,

      ClassDeclaration: skipper,

      ImportDeclaration: withMeta(function (node, _ref5) {
        var moduleImports = _ref5.moduleImports;

        moduleImports.push(node);
      }),

      InterfaceDeclaration: withMeta(function (node, _ref6) {
        var interfaces = _ref6.interfaces;

        interfaces.push(node);
      }),

      Program: {
        exit: withMeta(function (node, meta) {
          var output = (0, _generators.generate)(meta);

          if (!meta.dryRun) {
            (0, _fsUtils.ensureDir)(meta.outpath);
            _fs2['default'].writeFileSync(meta.outpath, output);
          }
        })
      }
    },

    pre: function pre(file) {
      //console.log(file.opts);
      var meta = {};
      var _file$opts = file.opts;
      var filename = _file$opts.filename;
      var moduleRoot = _file$opts.moduleRoot;
      var _file$opts$extra$dts = _file$opts.extra.dts;
      var packageName = _file$opts$extra$dts.packageName;
      var typings = _file$opts$extra$dts.typings;
      var _file$opts$extra$dts$suppressModulePath = _file$opts$extra$dts.suppressModulePath;
      var suppressModulePath = _file$opts$extra$dts$suppressModulePath === undefined ? false : _file$opts$extra$dts$suppressModulePath;
      var _file$opts$extra$dts$suppressComments = _file$opts$extra$dts.suppressComments;
      var suppressComments = _file$opts$extra$dts$suppressComments === undefined ? false : _file$opts$extra$dts$suppressComments;
      var _file$opts$extra$dts$ignoreMembers = _file$opts$extra$dts.ignoreMembers;
      var ignoreMembers = _file$opts$extra$dts$ignoreMembers === undefined ? /^_.*/ : _file$opts$extra$dts$ignoreMembers;
      var _file$opts$extra$dts$ignoreEmptyInterfaces = _file$opts$extra$dts.ignoreEmptyInterfaces;
      var ignoreEmptyInterfaces = _file$opts$extra$dts$ignoreEmptyInterfaces === undefined ? true : _file$opts$extra$dts$ignoreEmptyInterfaces;
      var _file$opts$extra$dts$ignoreEmptyClasses = _file$opts$extra$dts.ignoreEmptyClasses;
      var ignoreEmptyClasses = _file$opts$extra$dts$ignoreEmptyClasses === undefined ? false : _file$opts$extra$dts$ignoreEmptyClasses;
      var _file$opts$extra$dts$dryRun = _file$opts$extra$dts.dryRun;
      var dryRun = _file$opts$extra$dts$dryRun === undefined ? false : _file$opts$extra$dts$dryRun;

      var moduleId = packageName + '/' + (0, _path.relative)(moduleRoot, filename).replace('.js', '');
      meta.root = packageName;
      meta.moduleId = moduleId;
      meta.moduleExports = [];
      meta.moduleImports = [];
      meta.interfaces = [];
      meta.skipStack = [];
      meta.outpath = (0, _path.join)(typings, moduleId + '.d.ts');
      meta.suppressModulePath = suppressModulePath;
      meta.suppressComments = suppressComments;
      meta.ignoreMembers = ignoreMembers;
      meta.ignoreEmptyInterfaces = ignoreEmptyInterfaces;
      meta.ignoreEmptyClasses = ignoreEmptyClasses;
      meta.dryRun = dryRun;

      file.metadata[symb] = meta;
    }
  });
}
//# sourceMappingURL=plugin.js.map